@@ -10,6 +10,7 @@
 #' @param t_vector Numeric.  A vector of times at which to calculate PAF (only specified if model is coxph)
 #' @param ci_level Numeric.  Default 0.95. A number between 0 and 1 specifying the confidence level
 #' @param ci_type Character.  Defalt norm.  A vector specifying the types of confidence interval desired.  "norm", "basic", "perc" and "bca" are the available methods
+#' @param weight_vec An optional vector of inverse sampling weights for survey data (note that variance will not be calculated correctly if sampling isn't independent).  Note that this vector will be ignored if prev is specified, and the weights will be calibrated so that the weighted sample prevalence of disease equals prev.
 #' @references Bruzzi, P., Green, S.B., Byar, D.P., Brinton, L.A. and Schairer, C., 1985. Estimating the population attributable risk for multiple risk factors using case-control data. American journal of epidemiology, 122(5), pp.904-914
 #' @return A numeric estimated impact fraction if ci=FALSE, or for survival data a vector of estimated impact corresponding to event times in the data.  If ci=TRUE, a vector with elements corresponding to the raw estimated impact fraction, estimated bias, bias corrected estimate and lower and upper elements of any confidence procedures requested.  If ci=TRUE, and a coxph model is fit, a matrix will be returned, with rows corresponding to the times at which the impact fraction is calculated.
 #' @export
@@ -29,8 +30,7 @@
 #' data=stroke_reduced)
 #' impact_fraction(model=model_exercise,stroke_reduced,new_data,
 #' calculation_method = "B")
-impact_fraction <- function(model, data, new_data, calculation_method="B",prev=NULL,ci=FALSE,boot_rep=100,t_vector=NULL, ci_level=0.95, ci_type=c("norm")){
-
+impact_fraction <- function(model, data, new_data, calculation_method="B",prev=NULL,ci=FALSE,boot_rep=100,t_vector=NULL, ci_level=0.95, ci_type=c("norm"), weight_vec=NULL){
 
   if(!is.data.frame(data)){
     stop(
@@ -62,29 +62,18 @@ impact_fraction <- function(model, data, new_data, calculation_method="B",prev=N
   }
   response <- as.character(model$formula)[2]
 
-  model_type <- NULL
-  if(grepl("^glm$",as.character(model$call)[1],perl=TRUE)){
-    model_type <- "glm"
-       if (!as.character(model$family[1])=="binomial" & ! as.character(model$family[2]) %in% c("logit","log")) {
-      stop(
-        "The family must be binomial and link must be either log or logistic"
-      )
-    }
-  }
-  if(grepl("^coxph$",as.character(model$call)[1],perl=TRUE)){
-    if("userCall" %in% names(model)){
-      model_type <- "clogit"
+  model_type <- class(model)[1]
+
+  if(model_type=="clogit"){
       vars <- gsub(pattern=' ',replacement='',x=unlist(strsplit(as.character(model$call)[2],split="[~*+]")))
       vars <- gsub(pattern='^ns\\((.*),df=.*\\)$',replacement='\\1',x=vars)
       vars <- gsub(pattern='^ns\\((.*),knots=.*\\)$',replacement='\\1',x=vars)
       vars <- gsub(pattern='^strata\\((.*)\\)$',replacement='\\1',x=vars)
       vars <- gsub(pattern='^Surv\\(rep\\([0-9]*,[0-9]*\\),(.*)\\)$',replacement='\\1',x=vars)
       response <- vars[1]
-    }else{
-      model_type <- "coxph"
-    }
   }
-  if (is.null(model_type)) {
+
+  if (!model_type %in% c("glm","coxph","clogit")) {
     stop(
       "Model must be either a glm, conditional logistic regression or Cox-proportional hazards regression"
     )
@@ -125,24 +114,26 @@ impact_fraction <- function(model, data, new_data, calculation_method="B",prev=N
   N <- nrow(data)
   if(calculation_method=="B"){
 
-    if(!ci) return(if_bruzzi(data, ind=1:N, model=model,model_type=model_type,new_data=new_data,response=response))
+    if(!ci) return(if_bruzzi(data, ind=1:N, model=model,model_type=model_type,new_data=new_data,response=response, weight_vec=weight_vec))
     if(ci){
          nc <- options()$boot.ncpus
       cl <- parallel::makeCluster(nc)
-      parallel::clusterExport(cl, c("coxph","clogit","strata","Surv","ns"))
-            res <- boot::boot(data=data,statistic=if_bruzzi,R=boot_rep, model=model,model_type=model_type,new_data=new_data,response=response,cl=cl)
+      if("survival" %in% (.packages())) parallel::clusterExport(cl, c("coxph","clogit","strata","Surv"))
+      if("splines" %in% (.packages())) parallel::clusterExport(cl, c("ns"))
+            res <- boot::boot(data=data,statistic=if_bruzzi,R=boot_rep, model=model,model_type=model_type,new_data=new_data,response=response,cl=cl, weight_vec=weight_vec)
             parallel::stopCluster(cl)
            return(extract_ci(res=res,model_type=model_type,t_vector=t_vector,ci_level=ci_level,ci_type=ci_type))
     }
   }
   if(calculation_method=="D"){
 
-    if(!ci) return(if_direct(data,ind=1:N,model=model, model_type=model_type, new_data=new_data, prev=prev,t_vector=t_vector,response=response))
+    if(!ci) return(if_direct(data,ind=1:N,model=model, model_type=model_type, new_data=new_data, prev=prev,t_vector=t_vector,response=response, weight_vec=weight_vec))
     if(ci){
       nc <- options()$boot.ncpus
       cl <- parallel::makeCluster(nc)
-      parallel::clusterExport(cl, c("coxph","clogit","strata","Surv","ns"))
-       res <- boot::boot(data=data,statistic=if_direct,R=boot_rep,model=model, model_type=model_type, new_data=new_data, prev=prev,t_vector=t_vector,response=response,cl=cl)
+      if("survival" %in% (.packages())) parallel::clusterExport(cl, c("coxph","clogit","strata","Surv"))
+      if("splines" %in% (.packages())) parallel::clusterExport(cl, c("ns"))
+       res <- boot::boot(data=data,statistic=if_direct,R=boot_rep,model=model, model_type=model_type, new_data=new_data, prev=prev,t_vector=t_vector,response=response,cl=cl, weight_vec=weight_vec)
        parallel::stopCluster(cl)
        return(extract_ci(res=res,model_type=model_type,t_vector=t_vector,ci_level=ci_level,ci_type=ci_type))
     }
@@ -151,9 +142,22 @@ impact_fraction <- function(model, data, new_data, calculation_method="B",prev=N
 }
 
 
-if_bruzzi <- function(data,ind, model,model_type,  new_data,response){
+#' Internal:  Calculation of an impact fraction using the Bruzzi approach
+#'
+#' @param data A dataframe containing variables used for fitting the model
+#' @param ind  An indicator of which rows will be used from the dataset
+#' @param model Either a clogit or glm fitted model object.  Non-linear effects should be specified via ns(x, df=y), where ns is the natural spline function from the splines library.
+#' @param model_type Either a "clogit", "glm" or "coxph" model object
+#' @param new_data A dataframe (of the same variables and size as data) representing an alternative distribution of risk factors
+#' @param response A string representing the name of the outcome variable in data
+#' @param weight_vec An optional vector of inverse sampling weights
+#' @references Bruzzi, P., Green, S.B., Byar, D.P., Brinton, L.A. and Schairer, C., 1985. Estimating the population attributable risk for multiple risk factors using case-control data. American journal of epidemiology, 122(5), pp.904-914
+#' @return A numeric estimated impact fraction.
+#' @export
+if_bruzzi <- function(data,ind, model,model_type,  new_data,response, weight_vec){
 
 
+  if(is.null(weight_vec))   weight_vec = rep(1, nrow(data))
   N <- nrow(data)
 
   if(model_type == "clogit"){
@@ -187,24 +191,13 @@ if_bruzzi <- function(data,ind, model,model_type,  new_data,response){
         model_text <- gsub(pattern='^(.*)ns\\((.*)$',replacement='\\1splines::ns\\(\\2',x=model_text)
         stuff <- strsplit(model_text,split="splines::ns(",fixed=TRUE)
         model_text <- stuff[[1]][1]
-        thesplit <- paste0("splines::ns(",stuff[[1]][2],thesplit)
+              thesplit <- paste0("splines::ns(",stuff[[1]][2],thesplit)
       }
       model_text <- paste0(model_text,thesplit)
           model <- eval(parse(text=model_text))
 
     }
 
-    # clogit inherits predictions from coxph.  They seem strange at first but are equivalent to predictions from the following code which takes longer to run and so is commented out
-    #model$coefficients[is.na(model$coefficients)] <- 0
-    #them <- model.matrix(model)
-    #them[is.na(model.matrix(model))] <- 0
-    #eta1 <- them%*%model$coefficients
-    #the.mat <- model.matrix(as.formula(paste("~",gsub(paste('+ strata(',strataname,')',sep=''),'',x=as.character(model$formula[3]),fixed=TRUE),sep="")),data=new_data)
-    #the.mat <- the.mat[,-1,drop=FALSE]
-    #the.mat[is.na(the.mat)] <- 0
-    #eta2 <- the.mat%*%model$coefficients
-     #oldRR <- exp(eta1)
-    #newRR <- exp(eta2)
     oldRR <- predict(model,type="risk")
     newRR <- predict(model,type="risk",newdata=new_data)
     y <- data[,colnames(data)==response]
@@ -216,20 +209,9 @@ if_bruzzi <- function(data,ind, model,model_type,  new_data,response){
     if(!all(ind==(1:N))){
 
       data <- data[ind, ]
+       weight_vec <- weight_vec[ind]
       new_data <- new_data[ind, ]
-      model_text <- as.character(model$call)
-      model_text <- paste0("glm(",model_text[2],",data=data, family=binomial(link=",as.character(family(model)[2]),"))")
-      thesplit <- ""
-      while(length(grep(pattern='^.*ns\\(.*$',x=model_text))>0){
-        model_text <- gsub(pattern='^(.*)ns\\((.*)$',replacement='\\1splines::ns\\(\\2',x=model_text)
-        stuff <- strsplit(model_text,split="splines::ns(",fixed=TRUE)
-        model_text <- stuff[[1]][1]
-        thesplit <- paste0("splines::ns(",stuff[[1]][2],thesplit)
-      }
-      model_text <- paste0(model_text,thesplit)
-
-      model <- eval(parse(text=model_text))
-
+      model <- update(model,data=data)
     }
 
     # predict on linear predictor scale
@@ -237,13 +219,27 @@ if_bruzzi <- function(data,ind, model,model_type,  new_data,response){
     newRR <- exp(predict(model,newdata=new_data))
     y <- data[,colnames(data)==response]
   }
-
-  return(1 - mean(newRR[y==1]/oldRR[y==1]))
+  return(1 - sum(weight_vec[y==1]*(newRR[y==1]/oldRR[y==1]))/sum(weight_vec[y==1]))
 
 }
 
-if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,response){
 
+#' Internal:  Calculation of an impact fraction using the direct approach
+#'
+#' @param data A dataframe containing variables used for fitting the model
+#' @param ind  An indicator of which rows will be used from the dataset
+#' @param model Either a clogit, glm or coxph fitted model object.  Non-linear effects should be specified via ns(x, df=y), where ns is the natural spline function from the splines library.
+#' @param model_type Either a "clogit", "glm" or "coxph" model object
+#' @param new_data A dataframe (of the same variables and size as data) representing an alternative distribution of risk factors
+#' @param prev Population prevalence of disease (default NULL)
+#' @param t_vector A vector of times at which PAF estimates are desired (for a coxph model)
+#' @param response A string representing the name of the outcome variable in data
+#' @param weight_vec An optional vector of inverse sampling weights
+#' @return A numeric estimated impact fraction.
+#' @export
+if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,response,weight_vec){
+
+  if(is.null(weight_vec))   weight_vec = rep(1, nrow(data))
 
   N <- nrow(data)
   if(model_type == "coxph"){
@@ -251,18 +247,8 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
     if(!all(ind==(1:N))){
          data <- data[ind, ]
       new_data <- new_data[ind, ]
-      model_text <- as.character(model$call)
-      model_text <- paste0("survival::coxph(",model_text[2],",data=data)")
-     thesplit <- ""
-      while(length(grep(pattern='^.*ns\\(.*$',x=model_text))>0){
-        model_text <- gsub(pattern='^(.*)ns\\((.*)$',replacement='\\1splines::ns\\(\\2',x=model_text)
-        stuff <- strsplit(model_text,split="splines::ns(",fixed=TRUE)
-        model_text <- stuff[[1]][1]
-        thesplit <- paste0("splines::ns(",stuff[[1]][2],thesplit)
-      }
-      model_text <- paste0(model_text,thesplit)
-      model <- eval(parse(text=model_text))
-
+      weight_vec <- weight_vec[ind]
+      model <- update(model,data=data)
     }
        cum_haz <- survival::basehaz(model, centered=FALSE)
     t_indices <- integer(length(t_vector))
@@ -283,8 +269,6 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
 
   }
 
-
-
   add_term <- 0
 
   if(model_type=="clogit"){
@@ -310,9 +294,7 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
       new_data <- new_data[resamples,]
       # avoid duplication of strata names
       new_data[,colnames(data)==strataname] <- c(1:length(totake),1:length(totake))
-
       #refit model
-
       model_text <- paste0("survival::clogit(",model_text,",data=data)")
       thesplit=""
       while(length(grep(pattern='^.*ns\\(.*$',x=model_text))>0){
@@ -323,34 +305,24 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
       }
       model_text <- paste0(model_text,thesplit)
       model <- eval(parse(text=model_text))
-
+      #model <- update(model,data=data)
     }
-    # clogit inherits predictions from coxph.  They seem strange at first but are equivalent to predictions from the following code which takes longer to run and so is commented out
-    #model$coefficients[is.na(model$coefficients)] <- 0
-    #them <- model.matrix(model)
-    #them[is.na(model.matrix(model))] <- 0
-    #lp_old <- them%*%model$coefficients
-    #the.mat <- model.matrix(as.formula(paste("~",gsub(paste('+ strata(',strataname,')',sep=''),'',x=as.character(model$formula[3]),fixed=TRUE),sep="")),data=new_data)
-    #the.mat <- the.mat[,-1,drop=FALSE]
-    #the.mat[is.na(the.mat)] <- 0
-    #lp_new <- the.mat%*%model$coefficients
-    lp_old <- predict(model,newdata=data)
+       lp_old <- predict(model,newdata=data)
     lp_new <- predict(model, newdata=new_data)
     y <- data[,colnames(data)==response]
     N <- nrow(data)
-    weights <- rep(1, N)
     if(!is.null(prev)){
 
       data_prev <- mean(as.numeric(y==1))
-      weights[y==0] <- (1-prev)/(1-data_prev)
-      weights[y==1] <- prev/data_prev
+      weight_vec[y==0] <- (1-prev)/(1-data_prev)
+      weight_vec[y==1] <- prev/data_prev
 
     }
 
 
     if(!is.null(prev)){
 
-      temp_fun <- function(c){weighted.mean(exp(c+lp_old)/(1+exp(c+lp_old)),w=weights)-prev}
+      temp_fun <- function(c){weighted.mean(exp(c+lp_old)/(1+exp(c+lp_old)),w=weight_vec)-prev}
       add_term <- uniroot(temp_fun, interval=c(-100,100))$root
 
     }
@@ -364,21 +336,8 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
     if(!all(ind==(1:N))){
       data <- data[ind, ]
       new_data <- new_data[ind, ]
-      model_text <- as.character(model$call)
-      if(length(model_text)==4) model_text_u <- paste0("glm(",model_text[2],",data=data, family=binomial(link=",as.character(family(model)[2]),"))")
-      if(length(model_text)==5) model_text_u <- paste0("glm(",model_text[2],",data=data, family=binomial(link=",as.character(family(model)[2]),"),weights=",model_text[5],")")
-      model_text <- model_text_u
-      thesplit <- ""
-      while(length(grep(pattern='^.*ns\\(.*$',x=model_text))>0){
-        model_text <- gsub(pattern='^(.*)ns\\((.*)$',replacement='\\1splines::ns\\(\\2',x=model_text)
-        stuff <- strsplit(model_text,split="splines::ns(",fixed=TRUE)
-        model_text <- stuff[[1]][1]
-        thesplit <- paste0("splines::ns(",stuff[[1]][2],thesplit)
-      }
-      model_text <- paste0(model_text,thesplit)
-
-      model <- eval(parse(text=model_text))
-
+      weight_vec=weight_vec[ind]
+      model <- update(model,data=data)
     }
 
     lp_old <- predict(model,newdata=data)
@@ -386,21 +345,19 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
 
     y <- data[,colnames(data)==response]
   N <- nrow(data)
-    weights <- rep(1, N)
     if(!is.null(prev)){
 
       data_prev <- mean(as.numeric(y==1))
-      weights[y==0] <- (1-prev)/(1-data_prev)
-      weights[y==1] <- prev/data_prev
+      weight_vec[y==0] <- (1-prev)/(1-data_prev)
+      weight_vec[y==1] <- prev/data_prev
 
     }
 
-
     if(as.character(model$family[2])=="logit"){
 
       if(!is.null(prev)){
 
-        temp_fun <- function(c){weighted.mean(exp(c+lp_old)/(1+exp(c+lp_old)),w=weights)-prev}
+        temp_fun <- function(c){weighted.mean(exp(c+lp_old)/(1+exp(c+lp_old)),w=weight_vec)-prev}
         add_term <- uniroot(temp_fun, interval=c(-100,100))$root
 
       }
@@ -412,7 +369,7 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
 
         if(!is.null(prev)){
 
-          temp_fun <- function(c){weighted.mean(exp(c+lp_old),w=weights)-prev}
+          temp_fun <- function(c){weighted.mean(exp(c+lp_old),w=weight_vec)-prev}
           add_term <- uniroot(temp_fun, interval=c(-100,100))$root
 
         }
@@ -421,7 +378,7 @@ if_direct <- function(data, ind, model,model_type, new_data, prev,t_vector,respo
 
       }
   }
-  return((sum(weights*probs_old)-sum(weights*probs_new))/sum(weights*probs_old))
+  return((sum(weight_vec*probs_old)-sum(weight_vec*probs_new))/sum(weight_vec*probs_old))
 
 }
 
@@ -429,7 +386,7 @@ extract_ci <- function(res,model_type,t_vector,ci_level,ci_type,continuous=FALSE
 if(continuous){
 
   d <- data.frame(matrix(ncol=3 + 2*length(ci_type),nrow=length(res$t0)))
-  colnames(d) <- c("raw_estimate", "bias","bias_corrected",rep("",2*length(ci_type)))
+  colnames(d) <- c("est", "bias","debiased_est",rep("",2*length(ci_type)))
   for(i in 1:length(ci_type)) colnames(d)[(2+2*i):(3+2*i)] <- c(paste0(ci_type[i],"_lower"),paste0(ci_type[i],"_upper"))
 
 
@@ -454,7 +411,7 @@ if(continuous){
   if(model_type!="coxph"){
 
     v <- numeric(3 + 2*length(ci_type))
-    names(v) <- c("raw_estimate", "estimated_bias","bias_corrected_estimate",rep("",2*length(ci_type)))
+    names(v) <- c("est", "bias","debiased_est",rep("",2*length(ci_type)))
     for(i in 1:length(ci_type)) names(v)[(2+2*i):(3+2*i)] <- c(paste0(ci_type[i],"_lower"),paste0(ci_type[i],"_upper"))
     v[1] <- res$t0
     v[2] <- mean(res$t)-res$t0
@@ -473,7 +430,7 @@ if(continuous){
   if(model_type=="coxph"){
 
     d <- data.frame(matrix(ncol=3 + 2*length(ci_type),nrow=length(t_vector)))
-    colnames(d) <- c("raw_estimate", "estimated_bias","bias_corrected_estimate",rep("",2*length(ci_type)))
+    colnames(d) <- c("est", "bias","debiased_est",rep("",2*length(ci_type)))
     for(i in 1:length(ci_type)) colnames(d)[(2+2*i):(3+2*i)] <- c(paste0(ci_type[i],"_lower"),paste0(ci_type[i],"_upper"))
 
 
